@@ -20,6 +20,7 @@ logger = get_logger('rdfmts', '.rdfmts.log', file_and_console=True)
 
 CONFIG = MTCreationConfig()
 DEFAULT_OUTPUT_PATH = '/DeTrusty/Config/rdfmts.json'
+TYPING_WIKIDATA = '<http://www.wikidata.org/prop/direct/P31>'
 
 metas = [
     'http://www.w3.org/ns/sparql-service-description',
@@ -54,7 +55,9 @@ class Endpoint:
         return params_public
 
 
-def create_rdfmts(endpoints: list | dict, output: Optional[str] = DEFAULT_OUTPUT_PATH) -> Optional[JSONConfig]:
+def create_rdfmts(endpoints: list | dict,
+                  output: Optional[str] = DEFAULT_OUTPUT_PATH,
+                  interlinking: bool = False) -> Optional[JSONConfig]:
     logger_wrapper = get_logger('DeTrusty.Wrapper.RDFWrapper')
     logger_wrapper.setLevel(logging.WARNING)  # temporarily disable logging of contacting the source
 
@@ -103,35 +106,36 @@ def create_rdfmts(endpoints: list | dict, output: Optional[str] = DEFAULT_OUTPUT
             p.terminate()
 
     # now the interlinking
-    eofflags = []
-    epros = []
-    for e1 in endpoints:
-        for e2 in endpoints:
-            if e1 == e2:
-                continue
-            q = multiprocessing.Queue()
-            eofflags.append(q)
-            logger.info('Finding inter-links between: ' + e1.url + ' and ' + e2.url)
-            logger.info('==============================//=========//===============================')
-            p = multiprocessing.Process(target=_get_links, args=(e1, sparqlendps[e1.url], e2, sparqlendps[e2.url], q,))
-            epros.append(p)
-            p.start()
+    if interlinking:
+        eofflags = []
+        epros = []
+        for e1 in endpoints:
+            for e2 in endpoints:
+                if e1 == e2:
+                    continue
+                q = multiprocessing.Queue()
+                eofflags.append(q)
+                logger.info('Finding inter-links between: ' + e1.url + ' and ' + e2.url)
+                logger.info('==============================//=========//===============================')
+                p = multiprocessing.Process(target=_get_links, args=(e1, sparqlendps[e1.url], e2, sparqlendps[e2.url], q,))
+                epros.append(p)
+                p.start()
 
-    while len(eofflags) > 0:
-        for q in eofflags:
-            rdfmts = q.get()
-            for rdfmt in rdfmts:
-                rootType = rdfmt['rootType']
-                if rootType not in dsrdfmts:
-                    dsrdfmts[rootType] = rdfmt
-                else:
-                    _merge_mts(rdfmt, rootType, dsrdfmts)
-            eofflags.remove(q)
-            break
+        while len(eofflags) > 0:
+            for q in eofflags:
+                rdfmts = q.get()
+                for rdfmt in rdfmts:
+                    rootType = rdfmt['rootType']
+                    if rootType not in dsrdfmts:
+                        dsrdfmts[rootType] = rdfmt
+                    else:
+                        _merge_mts(rdfmt, rootType, dsrdfmts)
+                eofflags.remove(q)
+                break
 
-    for p in epros:
-        if p.is_alive():
-            p.terminate()
+        for p in epros:
+            if p.is_alive():
+                p.terminate()
 
     for e in sparqlendps:
         rdfmts = sparqlendps[e]
@@ -171,6 +175,8 @@ def _collect_rdfmts_from_source(endpoint: Endpoint, tq):
         linked_to = set()
         predicates = _get_predicates(endpoint, c)
         for p in predicates:
+            if 'wikiPageWikiLink' in p:
+                continue
             range_ = _get_predicate_range(endpoint, c, p)
             if len(range_) > 0:
                 [linked_to.add(r) for r in range_]
@@ -201,13 +207,23 @@ def _get_typed_concepts(endpoint):
     if endpoint.types:
         return endpoint.types
 
-    query = 'SELECT DISTINCT ?t WHERE { ?s a ?t . }'
+    query = 'SELECT DISTINCT ?t WHERE { ?s '
+    if 'wikidata' in endpoint.url:
+        query += TYPING_WIKIDATA
+    else:
+        query += 'a'
+    query += ' ?t . }'
     res_list, _ = _get_results_iter(query, endpoint)
     return [r['t'] for r in res_list if True not in [m in str(r['t']) for m in metas]]
 
 
 def _get_predicates(endpoint, type_):
-    query = 'SELECT DISTINCT ?p WHERE { ?s a <' + type_ + '> . ?s ?p ?pt . }'
+    query = 'SELECT DISTINCT ?p WHERE { ?s '
+    if 'wikidata' in endpoint.url:
+        query += TYPING_WIKIDATA
+    else:
+        query += 'a'
+    query += ' <' + type_ + '> . ?s ?p ?pt . }'
     res_list, status = _get_results_iter(query, endpoint)
     res_list = [r['p'] for r in res_list]
 
@@ -223,7 +239,12 @@ def _get_predicates(endpoint, type_):
 
 
 def _get_predicates_of_random_instances(endpoint, type_):
-    query = 'SELECT DISTINCT ?s WHERE { ?s a <' + type_ + '> . }'
+    query = 'SELECT DISTINCT ?s WHERE { ?s '
+    if 'wikidata' in endpoint.url:
+        query += TYPING_WIKIDATA
+    else:
+        query += 'a'
+    query += ' <' + type_ + '> . }'
     res_list, _ = _get_results_iter(query, endpoint, limit=100, max_tries=100, max_answers=100)
     results = []
     batches = [res_list[i:i+10] for i in range(0, len(res_list), 10)]
@@ -245,9 +266,14 @@ def _get_predicate_range(endpoint, type_, predicate):
 
     # second, get range from instances
     query = 'SELECT DISTINCT ?range WHERE {\n' \
-            '  ?s a <' + type_ + '> .\n' \
-            '  ?s <' + predicate + '> ?pt .\n' \
-            '  ?pt a ?range .\n}'
+            '  ?s '
+    if 'wikidata' in endpoint.url:
+        query += TYPING_WIKIDATA
+    else:
+        query += 'a'
+    query += ' <' + type_ + '> .\n' \
+             '  ?s <' + predicate + '> ?pt .\n' \
+             '  ?pt a ?range .\n}'
     res_list, _ = _get_results_iter(query, endpoint)
     ranges.extend([r['range'] for r in res_list if True not in [m in str(r['range']) for m in metas]])
 
@@ -331,7 +357,12 @@ def _get_links(endpoint1, rdfmt1, endpoint2, rdfmt2, q):
 
 
 def _get_external_links(endpoint1, root_type, predicate, endpoint2, rdfmt2):
-    query = 'SELECT DISTINCT ?o WHERE { ?s a <' + root_type + '> ; <' + predicate + '> ?o . FILTER (isIRI(?o)) }'
+    query = 'SELECT DISTINCT ?o WHERE { ?s '
+    if 'wikidata' in endpoint1.url:
+        query += TYPING_WIKIDATA
+    else:
+        query += 'a'
+    query += ' <' + root_type + '> ; <' + predicate + '> ?o . FILTER (isIRI(?o)) }'
     e1_objects, _ = _get_results_iter(query, endpoint1, max_tries=100)
     e1_objects = [obj['o'] for obj in e1_objects]
     batches = [e1_objects[i:i + 45] for i in range(0, len(e1_objects), 45)]
@@ -341,8 +372,13 @@ def _get_external_links(endpoint1, root_type, predicate, endpoint2, rdfmt2):
         query = 'SELECT DISTINCT ?c WHERE {\n' \
                 '  VALUES ?o { ' + ' '.join(batch) + ' }\n' \
                 '  ?s ?p ?o .\n' \
-                '  ?s a ?c .\n' \
-                '}'
+                '  ?s '
+        if 'wikidata' in endpoint2.url:
+            query += TYPING_WIKIDATA
+        else:
+            query += 'a'
+        query += ' ?c .\n' \
+                 '}'
         res_list, _ = _get_results_iter(query, endpoint2, max_tries=10)
         [linked_to.add(r['c']) for r in res_list]
 
